/* $Id: crires_model_kernel.h,v 1.27 2012-01-12 17:57:39 bristowp Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: bristowp $
 * $Date: 2012-01-12 17:57:39 $
 * $Revision: 1.27 $ 
 * $Name: not supported by cvs2svn $
 */

#ifndef CRIRES_MODEL_KERNEL_H
#define CRIRES_MODEL_KERNEL_H

/*-----------------------------------------------------------------------------
   								Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

/*-----------------------------------------------------------------------------
   							   Calibration Switch
 -----------------------------------------------------------------------------*/

#define cal_flag 0

/*-----------------------------------------------------------------------------
   								Define
 -----------------------------------------------------------------------------*/

#define chipxpix 1024.0
#define chipxspace 0.0
#define chipypix 512.0
/* (4*(chipxpix)+3*(chipxspace)) */
#define ASIZE 4096
#define BSIZE 512

#define fix_jaw_off 0.5;
#define is_buff 0.125;

/*-----------------------------------------------------------------------------
   								New types
 -----------------------------------------------------------------------------*/

typedef double DOUBLE;

typedef DOUBLE modes[60];
typedef int modes_i[60];
/* the following three lines are the only one you have to change to modify the vector/matrix default size  */
typedef DOUBLE vec[4];
typedef DOUBLE cvec[4];
typedef DOUBLE mat[4][4];
typedef int detloc[3];
typedef int XY[3];

typedef struct {
  DOUBLE best,min,max;
  int flag;
  char  name[20];
} ann_all_par;

typedef struct
  {
    int counter; 
    double x; 
    double y; 
    int chip; 
    double wave; 
    double flux;
    int slit_pos;
    int mode;
    int penc;
    int genc;
    double slitw;
  } coord ;

struct crires
{
  /* parameterlist */
  DOUBLE temper; /* T in K */
  DOUBLE mues, nues, taues, es_s, es_w, es_x, es_xf, es_x_off, es_y; /* for entrance slit */
  DOUBLE fcp; /* prism collimator */
  DOUBLE axis_mup, axis_nup, axis_taup; /* for prism axis */
  DOUBLE mup1, nup1, taup1; /* for prism crossdiperser entrance surface */
  DOUBLE penc_co1, penc_co2, penc_co3, genc_co1, genc_co2, genc_co3; /* coeffs for prism and grating encoder/angle relations */
  DOUBLE rind, rind_a1, rind_a2, rind_a3, rind_b1, rind_b2, rind_lm1, rind_lm2, rind_lm3, rind_wta0, rind_wta1, rind_wta2, rind_wtb0, rind_wtb1, rind_wtb2; /* for prism simple refrative index fit */
  DOUBLE rind_delta;/*extra param to allow ref_ind change in annealing*/
  DOUBLE mup2, nup2, taup2; /* for prism crossdiperser reflection surface */
  DOUBLE muis,nuis,tauis,flipisx,flipisy,fis,slitx,slity,slitrot,slitw,slith,xis,yis;/*Intermediate slit */
  int slit_on;/*Intermediate slit */
  DOUBLE axis_mug, axis_nug, axis_taug; /* for grating axis */
  DOUBLE mug, nug, taug, sg, grat_alpha, grat_beta; /* for grating */
  DOUBLE muk, nuk, tauk, fk, cam_dis, cam_ang; /* for cameramirror */
  DOUBLE mud, nud, taud, pix, offx, offy, flipx, flipy, xpospix, ypospix; /* for detector */
  int simpl, uvab;
  DOUBLE nx, ny, slitp;

  DOUBLE slitxup, slitxdown, slityup, slitydown; /* intermediate slit limits*/

  int onslit;

  detloc chippix; /*chip number, x pixel, y pixel*/

  double xdet,ydet; /*float x and y co-ordinate*/

  double det_array_x, det_array_y; /*centre of detector array in mm*/

  cvec chipx, chipy; /*Centres of each chip relative to detector centre in mm*/

  /*Limits for each detector chip*/
  cvec chipxup, chipyup, chipxdown, chipydown;
  cvec chiprot; /*relative rotation of each chip (deg)*/

  /* pin cushion distortion coeffs for each chip*/
  cvec pc_x_xx, pc_x_x1, pc_x_yy, pc_x_y1, pc_x_xy;
  cvec pc_y_xx, pc_y_x1, pc_y_yy, pc_y_y1, pc_y_xy;

  modes prism;
  modes grat;
  modes slit;
  modes_i ord;

/* matrices which can be computed without order, wavelength and slitposition */
  mat e_slit; /*rotation from entrance slit*/
  mat toprism1; /*rotation to first prism surface*/
  mat toprism2; /*rotation to second prism surface*/
  mat toprism3; /*rotation to third prism surface*/
  mat prism_out; /*rotation out of prism*/
  mat tograt; /*rotation to grating*/
  mat todetector; /*rotation to detector*/

};

/*-----------------------------------------------------------------------------
   							        Prototypes
 -----------------------------------------------------------------------------*/

int crires_model_init(struct crires*);
void crires_model_p_rot(struct crires*);
int crires_model_ref_ind_read(DOUBLE** znse_ref, DOUBLE temper);
void crires_model_eval(DOUBLE  lambda, int morder, DOUBLE** znse_ref, struct crires* p_crires);
void crires_model_detpix(struct crires* p_crires);

DOUBLE* crires_model_alloc1Darray(int asize); 
double** crires_model_alloc2Darray( int asize, int bsize);
double*** crires_model_alloc3Darray2(int asize,int bsize,int csize);
int** crires_model_alloc2Darray_i(int asize, int bsize);
float** crires_model_alloc2Darray_f(int asize, int bsize);
int crires_model_free2Darray(double** ccdtemp, int asize);
int crires_model_free3Darray(double*** ccdtemp, int asize,int bsize);
int crires_model_free2Darray_i(int** ccdtemp, int asize);
int crires_model_free2Darray_f(float** ccdtemp, int asize);
double* crires_model_copy2D_to_1D(double** ccdtemp, int asize, int bsize);
double** crires_model_copy1D_to_2D(double* oneDccdtemp, int asize, int bsize);

void crires_model_addvectors(vec, vec);  
void crires_model_addvectors3D(vec, vec);  
void crires_model_subtractvectors(vec,vec);
DOUBLE crires_model_scalarproduct(vec,vec);
void crires_model_multiply(vec a, DOUBLE k);
void crires_model_multiplythreematrix(mat A, mat B, mat C,mat D);
void crires_model_showvector(vec a);
void crires_model_nullvector(vec a);
void crires_model_copyvector(vec, vec);
void crires_model_normz(vec a);
void crires_model_normall(vec a);

void crires_model_showmatrix(mat A);
void crires_model_initializematrix(mat A);
void crires_model_multiplymatrix(mat A,mat B, mat C);
void crires_model_nullmatrix(mat A);
void crires_model_copymatrix(mat A, mat B);
void crires_model_matrixforvector(vec a,mat B,vec c);
void crires_model_transpose(mat A,mat B);

void crires_model_rotationmatrix(mat A, const char axis, const DOUBLE angle);
void crires_model_rotin(mat A,const DOUBLE x_angle, const DOUBLE y_angle,const DOUBLE z_angle);
void crires_model_refract(vec, DOUBLE, vec);
double crires_model_slitw_tab(int mm);
DOUBLE crires_model_wvlen(DOUBLE** detec,int order,int grat_enc,int prism_enc,DOUBLE T,int chip,DOUBLE x,DOUBLE y);

cpl_imagelist * crires_model_wavpix(const char*, const char*, int) ;

cpl_vector ** crires_model_profile(const char*, const char *, int, double) ;
cpl_vector ** crires_model_wavelengths(const char*, const char*, int,
        double, cpl_imagelist *) ;
double ** crires_model_predict(const char*, const char*, const char*, int);
cpl_table * crires_model_qanneal(const char*, coord* , int) ;
int qget_meas_coordinates(int, coord*, char*) ;
int qcountlines(const char*) ;
int crires_model_preset_angles(char*, struct crires*);
int crires_model_setup(const char * sci_file, const char * cfg_tab_file,
		       int order, struct crires * p_crires);
int crires_model_config_check(const char*   cri_config_file,
			      const char*   raw_file);
int temp_order_fn(int penc,int genc);
#endif
